#!/bin/sh

downloadPackage() 
{
    DOWNLOAD_RESULT="$1"
    DL_PLATFORM="$2"
    SOURCE_URL_HOME="$3"
    SOURCE_FILE="$4"
    DESTINATION_HOME="$5"
    CURRENT_DIR=`pwd`

    echo "Downloading package $SOURCE_FILE..."
    cd "$DESTINATION_HOME"
    if [ "$DL_PLATFORM" = "solaris" ]; then
        /usr/sfw/bin/wget --no-check-certificate "$SOURCE_URL_HOME/$SOURCE_FILE"
    else
        curl -k "$SOURCE_URL_HOME/$SOURCE_FILE" -o "$SOURCE_FILE"
    fi
    if [ -f "$SOURCE_FILE" ]; then
        echo "Download completed"
        eval $DOWNLOAD_RESULT="Y"
    else
        echo "File does not exist"
        eval $DOWNLOAD_RESULT="N"
    fi
    cd "$CURRENT_DIR"
}

checkValidUrl()
{
    RESULT_VALID_URL="$1"
    URL=`echo "$2" | tr [:upper:] [:lower:]`

    if [ `echo $URL | cut -c1-5` = "http:" -o `echo $URL | cut -c1-6` = "https:" ]; then
        eval $RESULT_VALID_URL="Y"
    else
        eval $RESULT_VALID_URL="N"
    fi
}

# extract server address from name in format obm-nix-%PORT%-%HOST_NAME%-%PROTOCOL%.sh, where %PROTOCOL% is "http"/"https"
parseHostAddrFromName()
{
    RESULT_ADDR="$1"
    # add prefix here to avoid failure case "-abc", prefix will be abandoned below
    NAME=`basename "prefix$2"`

    eval $RESULT_ADDR=""
    PROTOCOL=""

    while true; do
        # sed "s|^[^-]*$||": replace whole string with empty if whole string is without '-'
        if [ -z "`echo $NAME | sed \"s|^[^-]*$||\"`" ]; then
            break;
        fi
        # get start character only for protocol checking
        TMP_PROTOCOL=`echo "$NAME" | sed "s|^.*-||" | cut -c1-5`
        # remove postfix
        NAME=`echo "$NAME" | sed "s|-[^-]*$||"`
        if [ ! -z "`echo $TMP_PROTOCOL | grep \"^https\"`" ]; then
            PROTOCOL="https"
            break;
        fi
        if [ ! -z "`echo $TMP_PROTOCOL | grep \"^http\"`" ]; then
            PROTOCOL="http"
            break;
        fi
    done

    if [ ! -z "$PROTOCOL" ]; then
        # remove prefix
        NAME=`echo "$NAME" | sed "s|[^-]*-||"`
        while true; do
            # sed "s|^[^-]*$||": replace whole string with empty if whole string is without '-'
            if [ -z "`echo $NAME | sed \"s|^[^-]*$||\"`" ]; then
                break;
            fi
            # PORT must be either a number or empty
            PORT=`echo "$NAME" | cut -d'-' -f1 | sed "s|.*[^0-9].*||"`
            NAME=`echo "$NAME" | sed "s|[^-]*-||"`
            # check if PORT is valid
            if [ ! -z "$PORT" ] && [ "$PORT" -le 65535 ] && [ "$PORT" -ge 0 ]; then
                if [ ! -z "$NAME" ]; then
                    eval "$RESULT_ADDR=\"$PROTOCOL://$NAME:$PORT\""
                fi
                break;
            fi
        done
    fi
}

parseHostAddrFromRpm()
{
    RESULT_HOST_ADDR="$1"
    PARSE_HOST_PLATFORM="$2"
    ADDR=""

    ORIGINAL_IFS="$IFS"
    IFS=$'\n'
    eval $RESULT_HOST_ADDR=""
    SEARCH=".rpm"
    # for word in `ps ax | grep $SEARCH | grep -v grep`; do
    for word in `ps ax -ocommand | grep $SEARCH | grep -v grep`; do
        parseHostAddrFromName ADDR "$word"
        if [ ! -z "$ADDR" ]; then
            eval "$RESULT_HOST_ADDR=\"$ADDR\""
            break;
        fi
    done
    IFS="$ORIGINAL_IFS"
}

downloadJvmAndComponent() 
{
    LOCAL_RESULT="$1"
    MODE="$2"
    OS_ARCH="$3"
    PLATFORM="$4"
    TARGET="$5"
    HOSTADDR=""
    
    eval $LOCAL_RESULT="N"
    
    # ------------------------------ Parse host address ----------------------------------------
    if [ "$MODE" = "rpm" ]; then
        parseHostAddrFromRpm HOSTADDR "$PLATFORM"
    else
        parseHostAddrFromName HOSTADDR "$0"
        if [ -z "$HOSTADDR" ]; then
            echo "Please enter the server address (e.g. http://your.server:8080): "
            read HOSTADDR
        fi
    fi
    if [ -z "$HOSTADDR" ]; then
        echo "Host address is not defined"
        return
    fi
    IS_VALID_URL="N"
    checkValidUrl IS_VALID_URL "$HOSTADDR"
    if [ "$IS_VALID_URL" = "N" ]; then
        echo "Invalid host address"
        return
    fi
    
    # ------------------------------ Check jvm url ----------------------------------------
    JVM_URL="obs/download/jvm"
    if [ ! -z "$JVM_URL" ] ; then
        IS_VALID_URL="N"
        checkValidUrl IS_VALID_URL "$JVM_URL"
        if [ "$IS_VALID_URL" = "N" ]; then
            JVM_URL="$HOSTADDR/$JVM_URL"
        fi
    fi
    if [ -z "$JVM_URL" ] ; then
        echo "JVM package's URL is not defined"
        return
    fi
    
    # ------------------------------ Check component url ----------------------------------------
    COMPONENT_URL="obs/download/component"
    if [ ! -z "$COMPONENT_URL" ] ; then
        IS_VALID_URL="N"
        checkValidUrl IS_VALID_URL "$COMPONENT_URL"
        if [ "$IS_VALID_URL" = "N" ]; then
            COMPONENT_URL="$HOSTADDR/$COMPONENT_URL"
        fi
    fi
    if [ -z "$COMPONENT_URL" ] ; then
        echo "Component package's URL is not defined"
        return
    fi
    
    # ------------ Download jvm and components file from server ------------------
    # jvm
    PKG_JVM_FILE=""
    case "`uname`" in
        Linux*)
            if [ "$OS_ARCH" = "64" ]; then
                PKG_JVM_FILE="jre-std-linux-amd64.tar.gz"
            elif [ "$OS_ARCH" = "32" ]; then
                PKG_JVM_FILE="jre-std-linux-x86-586.tar.gz"
            fi
            ;;
        *)
            ;;
    esac
    if [ -z "$PKG_JVM_FILE" ]; then
        echo "No JVM package is defined"
    else
        for word in $PKG_JVM_FILE
        do
            downloadPackage DOWNLOAD_RESULT "$PLATFORM" "$JVM_URL" "$word" "$TARGET"
            if [ "$DOWNLOAD_RESULT" = "N" ] ; then
                echo "Failed to download JVM files."
                return
            fi
        done
    fi
    
    # component
    PKG_COMPONENT_FILE=""
    if [ "$OS_ARCH" = "64" ]; then
        PKG_COMPONENT_FILE="app-common.tar.gz app-native-nix-x64.tar.gz app-nix-obm.tar.gz aua-common.tar.gz aua-native-nix-x64.tar.gz aua-nix-obm.tar.gz util-common.tar.gz util-nix-obm.tar.gz properties-common.tar.gz app-inst-nix-obm.tar.gz aua-inst-nix-obm.tar.gz"
    elif [ "$OS_ARCH" = "32" ]; then
        PKG_COMPONENT_FILE="app-common.tar.gz app-native-nix-x86.tar.gz app-nix-obm.tar.gz aua-common.tar.gz aua-native-nix-x86.tar.gz aua-nix-obm.tar.gz util-common.tar.gz util-nix-obm.tar.gz properties-common.tar.gz app-inst-nix-obm.tar.gz aua-inst-nix-obm.tar.gz"
    else
        PKG_COMPONENT_FILE="app-common.tar.gz app-native-nix-others.tar.gz app-nix-obm.tar.gz aua-common.tar.gz aua-native-nix-others.tar.gz aua-nix-obm.tar.gz util-common.tar.gz util-nix-obm.tar.gz properties-common.tar.gz app-inst-nix-obm.tar.gz aua-inst-nix-obm.tar.gz"
    fi
    if [ -z "$PKG_COMPONENT_FILE" ]; then
        echo "No component package is defined"
    else
        for word in $PKG_COMPONENT_FILE
        do
            downloadPackage DOWNLOAD_RESULT "$PLATFORM" "$COMPONENT_URL" "$word" "$TARGET"
            if [ "$DOWNLOAD_RESULT" = "N" ] ; then
                echo "Failed to download component files."
                return
            fi
        done
    fi
    eval $LOCAL_RESULT="Y"
}

# Function to get string segment with seperator '='
# e.g. $1 = "APP_HOME=/usr/local/obm APP_HOME=/root/Documents/obm" , $2 = 3 
# then this function returns /root/Documents/obm
get_column_value()
{
    STRING_PATTERN="$1"
    COL_NUM=$2
    echo $STRING_PATTERN | awk -F"=" '{print $'"${COL_NUM}"'}'
}

# ------------------------- Retrieve APP_HOME ----------------------------------
cd `dirname "$0"`

SCRIPT_NAME="$0"
MODE="manual"
if [ ! -z "$1" ]; then
    if [ "$1" = "rpm" ]; then
        MODE="rpm"
    fi
fi
APP_HOME=`pwd`
if [ ! -z "$2" ]; then
    APP_HOME="$2"
fi
PRIVILEGE_CHK=""
if [ ! -z "$3" ]; then
    PRIVILEGE_CHK="$3"
fi
USER_JVM_HOME=""
APP_BIN=$APP_HOME/bin
TAR_FOLDER_NAME=_components
TAR_FOLDER="$APP_HOME/$TAR_FOLDER_NAME"

# get arch of this system
PLATFORM_NAME=""
case "`uname`" in
    Solaris* | SunOS*)
        PLATFORM_NAME="solaris"
        case "`isainfo -b`" in
            32 | 64)
                OS_ARCH="32"
                ;;
            # amd64 is NOT support in Solaris / SunOS
            #64)
            #    OS_ARCH="64"
            #    ;;
            *)
                OS_ARCH="Others"
                ;;
        esac
        ;;
    *)
        PLATFORM_NAME="others"
        case "`uname -m`" in
            i[3-6]86)
                OS_ARCH="32"
                ;;
            x86_64 | amd64)
                OS_ARCH="64"
                ;;
            *)
                OS_ARCH="Others"
                ;;
        esac
        ;;
esac

# Get INIT_PATH to get APP_HOME from running service
case "`uname`" in 
    Linux*)      INSTALL_TYPE=NIX ;;
    Solaris*)    INSTALL_TYPE=NIX ;;
    SunOS*)      INSTALL_TYPE=NIX ;;
    OpenBSD*)    INSTALL_TYPE=OPENBSD ;;
    *BSD*)       INSTALL_TYPE=BSD ;;
    **)          INSTALL_TYPE=NOT_SUPPORTED ;;
esac

# Initialize INIT_PATH to empty
INIT_PATH=""
# Set INIT_PATH for different OS
if [ $INSTALL_TYPE = "NIX" ]; then
    if [ -z "$INIT_PATH" ]; then
        if [ -d /etc/init.d ]; then
            INIT_PATH=/etc/init.d
        elif [ -d /etc/rc.d/init.d ]; then
            INIT_PATH=/etc/rc.d/init.d
        fi
    fi
    
elif [ $INSTALL_TYPE = "BSD" ]; then
    INIT_PATH=/usr/local/etc/rc.d
    
elif [ $INSTALL_TYPE = "OPENBSD" ]; then
    # get OpenBSD version
    OPENBSD_VERSION=`uname -r | cut -f1 -d"."`

    if [ $OPENBSD_VERSION -ge 5 ]
    then
        INIT_PATH=/etc/rc.d
    else
        INIT_PATH=/etc
    fi
fi

# -------------------- Print Logging Message Header ----------------------------
echo "Log Time: `date`"

# ------------------------------ Download jvm and components ------------------------------
if [ -d "$TAR_FOLDER" ]; then
    rm -rf "$TAR_FOLDER"
fi
mkdir "$TAR_FOLDER"

downloadJvmAndComponent RESULT "$MODE" "$OS_ARCH" "$PLATFORM_NAME" "$TAR_FOLDER"
if [ "$RESULT" = "N" ] ; then
    echo "Error in downloading files, installer will terminate"
    rm -rf "$TAR_FOLDER"
    exit 1
fi

# ------------ Backup user setting files ------------------
# Get old APP_HOME
if [ -f "$APP_HOME/bin/uninstall.sh" ]; then
    APP_HOME_OLD=$APP_HOME
elif [ ! -z "$INIT_PATH" ]; then
    OBM_SCRIPT_FILE="$INIT_PATH/obmscheduler"
    if [ -f $OBM_SCRIPT_FILE ] ; then
        OBM_HOME_INFO=`cat $OBM_SCRIPT_FILE | grep APP_HOME=`
        APP_HOME_OLD=`get_column_value "$OBM_HOME_INFO" 3`
    fi
fi

if [ -z "$APP_HOME_OLD" ]; then
    echo "  No old application found, begin fresh install"
else
    if [ -f "$APP_HOME_OLD/bin/uninstall.sh" ]; then
        echo "  Backup user setting files"

        # directory for temporary keeping backup files
        TEMP_DIR="_tmp"
        BACKUP_DIR="$APP_HOME/$TEMP_DIR"
        if [ -d "$BACKUP_DIR" ]; then
            echo "  Remove previous cache"
            rm -rf "$BACKUP_DIR"
        fi
        mkdir "$BACKUP_DIR"

        # directory to be backup / restore
        DIR_NAME="log"
        for DIR in $DIR_NAME; do
            if [ -d "$APP_HOME_OLD/$DIR" ]; then
                echo "    Backup $DIR"
                mv -f "$APP_HOME_OLD/$DIR" "$BACKUP_DIR/$DIR"
            fi
        done

        # file to be backup / restore
        FILE_NAME="home.txt config.ini afc.opt cb.opt"
        for FILE in $FILE_NAME; do
            if [ -f "$APP_HOME_OLD/$FILE" ]; then
                echo "    Backup $FILE"
                mv -f "$APP_HOME_OLD/$FILE" "$BACKUP_DIR/$FILE"
            fi
        done
        echo "  Backup finished"

        # Run uninstall to stop Scheduler, AUA services
        echo "  Uninstall previous version..."
        "$APP_HOME_OLD/bin/uninstall.sh" "no_privilege_check"

        echo "  Remove previous application files"
        APP_DATA="`ls "$APP_HOME_OLD" | sort `"
        for DATA in $APP_DATA; do
            if [ "$DATA" != "$TEMP_DIR" ] && [ "$DATA" != "$TAR_FOLDER_NAME" ] ; then
                if [ -d "$APP_HOME_OLD/$DATA" ] || [ -L "$APP_HOME_OLD/$DATA" ]; then
                    echo "    Remove directory $DATA"
                    rm -rf "$APP_HOME_OLD/$DATA"
                else
                    if [ -z "`echo \"$APP_HOME_OLD/$DATA\" | grep ${SCRIPT_NAME}$`" ]; then
                        if [ -f "$APP_HOME_OLD/$DATA" ]; then
                            echo "    Remove file $DATA"
                            rm "$APP_HOME_OLD/$DATA"
                        fi
                    fi
                fi
            fi
        done
        echo "  Remove application files finished"
    else
        echo "  No old application found, begin fresh install"
    fi
fi

# ----------------------- Untar jvm and component file --------------------------------
cd "$TAR_FOLDER"
for entry in "$TAR_FOLDER"/*
do
    case "$entry" in
        *"/jre-"* )
            if [ ! -d "$APP_HOME/jvm" ] ; then
                echo "$APP_HOME/jvm does not exist, creating this directory"
                mkdir "$APP_HOME/jvm" > /dev/null
            fi
            echo "Untar JVM file : $entry"
            if [ "$PLATFORM_NAME" = "solaris" ] ; then
                cd "$APP_HOME/jvm"
                gzip -cdf "$entry" | /usr/sfw/bin/gtar xf - > /dev/null
                cd "$TAR_FOLDER"
            elif [ "$PLATFORM_NAME" = "others" ] ; then
                tar -xzf "$entry" -C "$APP_HOME/jvm" > /dev/null
            fi
            ;;
        *)
            echo "Untar component file : $entry"
            if [ "$PLATFORM_NAME" = "solaris" ] ; then
                cd "$APP_HOME"
                gzip -cdf "$entry" | /usr/sfw/bin/gtar xf - > /dev/null
                cd "$TAR_FOLDER"
            elif [ "$PLATFORM_NAME" = "others" ] ; then
                tar -xzf "$entry" -C "$APP_HOME" > /dev/null
            fi
            ;;
    esac
done

cd "$APP_HOME"
echo "Untar jvm and component files finished"
rm -rf "$TAR_FOLDER"

# ----------------------- Verify if JVM home exist --------------------------------
JVM_HOME=$APP_HOME/jvm
if [ ! -d "$JVM_HOME" ]; then
    if [ -z "$USER_JVM_HOME" ]; then
        echo "Please enter your java 1.7 home: "
        read USER_JVM_HOME
    fi
    if [ -z "$USER_JVM_HOME" ]; then
        echo "Invalid java home!"
        exit 1
    fi
    if [ ! -d "$USER_JVM_HOME" ]; then
        echo "$USER_JVM_HOME does not exist!"
        exit 1
    fi
    JVM_SECURITY="$APP_HOME/util/jvm/lib/security"
    if [ -d "$JVM_SECURITY" ]; then
        echo "Copy java 1.7 from $USER_JVM_HOME"
        cp -rf "$USER_JVM_HOME" "$JVM_HOME"
        cp -rf $JVM_SECURITY/* $JVM_HOME/lib/security
    else
        echo "Create symbolic link from $USER_JVM_HOME"
        ln -sf "$USER_JVM_HOME" "$JVM_HOME"
    fi
fi

# ----------------------- Install Application --------------------------------
TIMESTAMP_LOG=`date '+%y%m%d%H%M%S'`
INSTALL_LOG="$APP_HOME/install-$TIMESTAMP_LOG.log"
INSTALL_SCRIPT="$APP_BIN/install.sh"
if [ ! -f "$INSTALL_SCRIPT" ]; then
    echo "Install script not exist"
    exit 1
fi
echo "Install Application Path: $APP_HOME"
sh "$INSTALL_SCRIPT" "$PRIVILEGE_CHK" >> "$INSTALL_LOG"

# ----------------------- Restore user data files --------------------------------
# directory for temporary keeping backup files
TEMP_DIR="_tmp"
BACKUP_DIR="$APP_HOME/$TEMP_DIR"

if [ -d "$BACKUP_DIR" ]; then
    echo "  Restore Previous Setting backup ..."

    # directory to be backup / restore
    DIR_NAME="log"
    for DIR in $DIR_NAME; do
        if [ -d "$BACKUP_DIR/$DIR" ]; then
            echo "    Restore $DIR"
            mv -f "$BACKUP_DIR/$DIR" "$APP_HOME/$DIR"
        fi
    done

    # migration logic for properties file 'afc.opt' and 'cb.opt'.
    FILEPATH_UTIL="$APP_HOME/util/lib/installer-util.jar"
    CLASSNAME_UTIL=PropertiesMigration

    # file to be backup / restore
    FILE_NAME="home.txt config.ini afc.opt cb.opt"
    for FILE in $FILE_NAME; do
        if [ -f "$BACKUP_DIR/$FILE" ]; then
            # afc.opt AND cb.opt need perform migration
            if [ "$FILE" = "afc.opt" ]; then
                echo "    Restore application opt file (afc.opt)"
                FILEPATH_OUTPUT="$BACKUP_DIR/afc.opt.new"
                FILEPATH_DEFAULT="$BACKUP_DIR/afc.opt"
                FILEPATH_CUSTOM="$APP_HOME/afc.opt"
                java -cp "${FILEPATH_UTIL}" "${CLASSNAME_UTIL}" "${FILEPATH_OUTPUT}" "${FILEPATH_DEFAULT}" "${FILEPATH_CUSTOM}"
                mv -f "${FILEPATH_OUTPUT}" "${FILEPATH_CUSTOM}"
            else
                if [ "$FILE" = "cb.opt" ]; then
                    echo "    Restore application opt file (cb.opt)"
                    FILEPATH_OUTPUT="$BACKUP_DIR/cb.opt.new"
                    FILEPATH_DEFAULT="$BACKUP_DIR/cb.opt"
                    FILEPATH_CUSTOM="$APP_HOME/cb.opt"
                    java -cp "${FILEPATH_UTIL}" "${CLASSNAME_UTIL}" "${FILEPATH_OUTPUT}" "${FILEPATH_DEFAULT}" "${FILEPATH_CUSTOM}"
                    mv -f "${FILEPATH_OUTPUT}" "${FILEPATH_CUSTOM}"
                else
                    echo "    Restore $FILE"
                    mv -f "$BACKUP_DIR/$FILE" "$APP_HOME/$FILE"
                fi
            fi
        fi
    done

    rm -rf "$BACKUP_DIR"
    echo "  Previous Setting backup restored"
fi

echo "Done"
exit 0
